/*******************************************************************************

WePhoto - Photo viewer for multi-touch devices
Copyright (C) 2010 Mihai Paslariu
Original Name MihPhoto
Adapted for WeTab 2011 by Peter Herzog www.pherzog.net

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "ImageLoadThread.h"
#include "Config.h"
#include <QFile>
#include <QDir>
#include <QImageReader>
#include <math.h>

void ImageLoadThread::run()
{
	while ( !_finished )
	{
		ImageLoadItem ili = _in.pop();
		if ( _finished ) break;

		QImage * img = _loadImage( ili.name, ili.w, ili.h );
		double zoom = _computeFitZoom( img, ili.w, ili.h );

		*ili.zoom = zoom;
		*ili.destination = img;

		_load_mutex.lock();
		_load--;
		_load_mutex.unlock();
	}
}

QImage * ImageLoadThread::_loadImage( QString name, int area_width, int area_height )
{
	QString fullname = g_config.current_dir + QDir::separator() + name;

	// get exif rotation if needed
	qreal rotation = 0;
	if ( g_config.rotate_by_exif )
		rotation = _getExifRotation( fullname );

	// create image reader
	QImageReader * reader = new QImageReader( fullname );
	QSize size = reader->size();

	// limit the size of the image on Symbian
	(void)area_width;
	(void)area_height;


	// actually load the image
	QImage * img = new QImage(size, reader->imageFormat());
	reader->setScaledSize( size );
	if ( !reader->read(img) )
	{
		delete reader;
		delete img;
		return NULL;
	}
	delete reader;

	// rotate the image
	QImage * img2 = img;
	if ( rotation != 0.0 )
	{
		QTransform tr;
		tr.rotate(rotation);
		img2 = new QImage();
		*img2 = img->transformed(tr);
		delete img;
	}
	return img2;
}

double ImageLoadThread::_computeFitZoom( QImage * img, int w, int h )
{
	if ( img == NULL ) return 1.0f;
	double xz = (double)w / (double)img->width();
	double yz = (double)h / (double)img->height();
	double z = ( xz < yz ? xz : yz );
	return z > 1.0f ? 1.0f : z;
}

qreal ImageLoadThread::_getExifRotation( QString fullname )
{
	// only check exif for jpg files
	if ( !fullname.endsWith(".jpg", Qt::CaseInsensitive) )
		return 0.0;

	// try to load exif orientation
	QFile f( fullname );
	quint16 orientation = 1;
	if ( f.open(QIODevice::ReadOnly))
	{
		printf("geöffnete Datei %s\n", fullname.toUtf8().data());
		// find exif start marker

		unsigned short w;
		bool in_exif = false;
		quint16 type = 0;
		quint32 len = 0;
		while ( f.read((char*)&w, sizeof(w)) )
		{
			if ( in_exif )
			{
				if ( (w & 0x00FF) == 0xFF ) {
					in_exif = false;
				} else if ( w == 0x0112 ) {
					// HACK
					f.read((char*)&type, sizeof(type)) > 0
						&& f.read((char*)&len, sizeof(len)) > 0
						&& f.read((char*)&orientation, sizeof(orientation)) > 0;
					break;
				}
			} else {
				if ( w == 0xE1FF )
				{
					in_exif = true;
				}
			}
		}

		f.close();
	}

	qreal rotation = 0;
	switch ( orientation )
	{
		case 8: rotation = -90; break;
		case 6: rotation = 90; break;
		default: break;
	}
	printf("Rotation %u-> %d\n", (unsigned int)orientation, (int)rotation);
	return rotation;
}
