/*******************************************************************************

WePhoto - Photo viewer for multi-touch devices
Copyright (C) 2010 Mihai Paslariu
Original Name MihPhoto
Adapted for WeTab 2011 by Peter Herzog www.pherzog.net

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <QtGui>
#include <stdio.h>

#include "MainWindow.h"
#include "ImageArea.h"
#include "Config.h"

MainWindow::MainWindow( QString startfile, bool fullscreen )
{
	setWindowTitle(tr("WePhoto speziell fuer das Wetab und die Community"));
	resize(1200, 700);
	_maximized = isMaximized();

	_image_area = new ImageArea;
	setCentralWidget(_image_area);
	_image_area->setFocusPolicy(Qt::StrongFocus);
	connect( _image_area, SIGNAL(closeOnTouch()), this, SLOT(close()) );
	connect( _image_area, SIGNAL(changeFullscreen()), this, SLOT(changeFullscreen()) );
	connect( _image_area, SIGNAL(loadFile()), this, SLOT(open()));
	_createActions();
	_createMenus();

	if ( fullscreen )
	{
		this->showFullScreen();
		menuBar()->hide();
	} else {
		this->show();
		_maximized = isMaximized();
	}

	if ( !startfile.isEmpty() )
	{
		_loadFiles( startfile );
	}
}

void MainWindow::closeEvent( QCloseEvent * event )
{
	event->ignore();
	g_config.save();
	exit(0);
}

void MainWindow::keyPressEvent( QKeyEvent * event )
{
	switch ( event->key() )
	{
		// non-configurable keys
		case Qt::Key_Escape:
			close();
			break;
		case Qt::Key_F:
			changeFullscreen();
			break;
		case Qt::Key_Space:
			_image_area->forward();
			break;
		case Qt::Key_Backspace:
			_image_area->back();
			break;
		case Qt::Key_Plus:
			_image_area->zoomIn();
			break;
		case Qt::Key_Minus:
			_image_area->zoomOut();
			break;
		case Qt::Key_BracketLeft:
			_image_area->rotateRight();
			break;
		case Qt::Key_BracketRight:
			_image_area->rotateLeft();
			break;
		case Qt::Key_Slash:
			_image_area->zoomToFit();
			break;
		case Qt::Key_0:
			_image_area->zoomToOriginalSize();
			break;
		case Qt::Key_Left:
			if ( event->modifiers() & Qt::ShiftModifier )
				_image_area->moveView(-50,0);
			else
				_image_area->back();
			break;
		case Qt::Key_Right:
			if ( event->modifiers() & Qt::ShiftModifier )
				_image_area->moveView(50,0);
			else
				_image_area->forward();
			break;
		case Qt::Key_Up:
			if ( event->modifiers() & Qt::ShiftModifier )
				_image_area->moveView(0,-50);
			break;
		case Qt::Key_Down:
			if ( event->modifiers() & Qt::ShiftModifier )
				_image_area->moveView(0,50);
			break;
		default:
			break;
	}
}

void MainWindow::wheelEvent ( QWheelEvent * event )
{
	if ( event->delta() > 0 )
		_image_area->back();
	else
		_image_area->forward();
}

void MainWindow::open()
{
	QString filename = QFileDialog::getOpenFileName(this,
		tr("Oeffne Bild"), g_config.last_open_dir, tr("Bilddateien (*.png *.jpg *.bmp *.gif)"));
	if ( !filename.isNull() )
		_loadFiles( filename );
}

void MainWindow::about()
{
	QString license = "<p>This program is free software: you can redistribute it and/or modify"
			  "it under the terms of the GNU General Public License as published by"
			  "the Free Software Foundation, either version 3 of the License, or"
			  "(at your option) any later version."
			  "<br /><br />"
			  "This program is distributed in the hope that it will be useful,"
			  "but WITHOUT ANY WARRANTY; without even the implied warranty of"
			  "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
			  "GNU General Public License for more details."
			  "<br /><br />"
			  "You should have received a copy of the GNU General Public License"
			  "along with this program. If not, see http://www.gnu.org/licenses/</p>";

	QString name = "<p><b>WePhoto</b> - zeige Fotos eines Verzeichnisses ganz einfach am Touchscreen.<br />"
		       "Copyright (C) 2010-2011 Mihai Paslariu<br>"
		       "Angepasst an das WeTab Feb 2011 fuer die WeTab-Community<br>"
		       "Peter Herzog alias PMaler (www.pherzog.net)</p>";

	QString version;
	version.sprintf("<p>Version %.2lf</p>", VERSION );

	QMessageBox::about(this, tr("WePhoto"), name + version + license );
}

void MainWindow::changeFullscreen()
{
	if ( this->isFullScreen() )
	{
		if ( _maximized )
			this->showMaximized();
		else
			this->showNormal();
		menuBar()->show();
	} else {
		_maximized = this->isMaximized();
		this->showFullScreen();
		menuBar()->hide();
	}
}

void MainWindow::showOrHideName()
{
	g_config.show_file_name = !g_config.show_file_name;
	_image_area->update();
}

void MainWindow::allowRotation()
{
	g_config.allow_rotation = !g_config.allow_rotation;
}

void MainWindow::rotateOnExif()
{
	g_config.rotate_by_exif = !g_config.rotate_by_exif;
}

void MainWindow::fullscreenOnDoubleClick()
{
	g_config.double_click_fullscreen = !g_config.double_click_fullscreen;
}
void MainWindow::dragspeed50()
{
    g_config.dragspeed = 50;
    _dragspeed250->setChecked( false );
    _dragspeed450->setChecked( false );
    _dragspeed1000->setChecked( false );
}
void MainWindow::dragspeed250()
{
    _dragspeed50->setChecked( false );
    g_config.dragspeed = 250;
    _dragspeed450->setChecked( false );
    _dragspeed1000->setChecked( false );
}
void MainWindow::dragspeed450()
{
    _dragspeed50->setChecked( false );
    g_config.dragspeed = 450;
    _dragspeed250->setChecked( false );
    _dragspeed1000->setChecked( false );
}
void MainWindow::dragspeed1000()
{
    _dragspeed50->setChecked( false );
    g_config.dragspeed = 1000;
    _dragspeed250->setChecked( false );
    _dragspeed450->setChecked( false );
}
void MainWindow::changeBackground()
{
	QColor c = QColorDialog::getColor( g_config.background, this );
	if ( c.isValid() )
	{
		g_config.background = c;
		_image_area->update();
	}
}

void MainWindow::_createActions()
{
	_open_act = new QAction(tr("&Oeffnen..."), this);
	_open_act->setShortcut(tr("Ctrl+O"));
	connect(_open_act, SIGNAL(triggered()), this, SLOT(open()));

	_fullscreen_act = new QAction(tr("&Fullscreen"), this);
	_fullscreen_act->setShortcut(tr("Ctrl+F"));
	connect(_fullscreen_act, SIGNAL(triggered()), this, SLOT(changeFullscreen()));

	_show_name_act = new QAction(tr("Zeige Namen"), this);
	_show_name_act->setCheckable(true);
	_show_name_act->setChecked( g_config.show_file_name );
	connect(_show_name_act, SIGNAL(triggered()), this, SLOT(showOrHideName()));

	_allow_rotation_act = new QAction(tr("Erl&aube Rotation mit zwei Finger"), this);
	_allow_rotation_act->setCheckable(true);
	_allow_rotation_act->setChecked( g_config.allow_rotation );
	connect(_allow_rotation_act, SIGNAL(triggered()), this, SLOT(allowRotation()));

	_rotate_exif_act = new QAction(tr("Rotiere bei EXIF Orientierung"), this);
	_rotate_exif_act->setCheckable(true);
	_rotate_exif_act->setChecked( g_config.rotate_by_exif );
	connect(_rotate_exif_act, SIGNAL(triggered()), this, SLOT(rotateOnExif()));

	_double_click_fullscreen_act = new QAction(tr("Fullscreen bei Doppelclick"), this);
	_double_click_fullscreen_act->setCheckable(true);
	_double_click_fullscreen_act->setChecked( g_config.double_click_fullscreen );
	connect(_double_click_fullscreen_act, SIGNAL(triggered()), this, SLOT(fullscreenOnDoubleClick()));

	_back_color_act = new QAction(tr("Hintergrund Farbe ..."), this);
	connect(_back_color_act, SIGNAL(triggered()), this, SLOT(changeBackground()));

	_dragspeed50 = new QAction(tr("Bildwechsel um 50Px"), this);
	_dragspeed50->setCheckable(true);
	_dragspeed50->setChecked( g_config.dragspeed <= 200 );
	connect(_dragspeed50, SIGNAL(triggered()), this, SLOT(dragspeed50()));

	_dragspeed250 = new QAction(tr("Bildwechsel um 250Px"), this);
	_dragspeed250->setCheckable(true);
	_dragspeed250->setChecked( g_config.dragspeed > 200 && g_config.dragspeed <= 400);
	connect(_dragspeed250, SIGNAL(triggered()), this, SLOT(dragspeed250()));

	_dragspeed450 = new QAction(tr("Bildwechsel um 450Px"), this);
	_dragspeed450->setCheckable(true);
	_dragspeed450->setChecked( g_config.dragspeed > 400 && g_config.dragspeed <= 850 );
	connect(_dragspeed450, SIGNAL(triggered()), this, SLOT(dragspeed450()));

	_dragspeed1000 = new QAction(tr("Bildwechsel um 1000Px"), this);
	_dragspeed1000->setCheckable(true);
	_dragspeed1000->setChecked( g_config.dragspeed >= 850 );
	connect(_dragspeed1000, SIGNAL(triggered()), this, SLOT(dragspeed1000()));


	_exit_act = new QAction(tr("E&xit"), this);
	_exit_act->setShortcut(tr("Ctrl+Q"));
	connect(_exit_act, SIGNAL(triggered()), this, SLOT(close()));

	_about_act = new QAction(tr("&Ueber"), this);
	connect(_about_act, SIGNAL(triggered()), this, SLOT(about()));
}

void MainWindow::_createMenus()
{
	_file_menu = new QMenu(tr("&Datei"), this);
	_file_menu->addAction(_open_act);
	_file_menu->addSeparator();
	_file_menu->addAction(_exit_act);

	_options_menu = new QMenu(tr("&Optionen"), this);
	_options_menu->addAction(_fullscreen_act);
	_options_menu->addAction(_show_name_act);
	if ( g_config.multitouch )
		_options_menu->addAction(_allow_rotation_act);
	_options_menu->addAction(_rotate_exif_act);
	_options_menu->addAction(_double_click_fullscreen_act);
	_options_menu->addAction(_back_color_act);
	_options_menu->addAction(_dragspeed50);
	_options_menu->addAction(_dragspeed250);
	_options_menu->addAction(_dragspeed450);
	_options_menu->addAction(_dragspeed1000);

	_help_menu = new QMenu(tr("&Hilfe"), this);
	_help_menu->addAction(_about_act);

	menuBar()->addMenu(_file_menu);
	menuBar()->addMenu(_options_menu);
	menuBar()->addMenu(_help_menu);
}

void MainWindow::_loadFiles( QString name )
{
	QString file_name = "";
	QString dir_name = QDir::currentPath();

	QFileInfo d(name);
	name = d.absoluteFilePath();
	if ( d.isDir() )
	{
		dir_name = name;
	} else {
		int i = name.lastIndexOf("/");
		if ( i > 0 )
		{
			dir_name = name.mid(0,i);
			file_name = name.mid(i+1);
		} else {
			file_name = name;
		}
	}

	printf("Oeffne Datei %s\n", file_name.toAscii().data());
	printf("Oeffne Verz. %s\n", dir_name.toAscii().data());
	//QMessageBox::information(this, "Info", file_name + ", " + dir_name + ", " + g_config.last_open_dir );
	_loadFiles( dir_name, file_name );
}

void MainWindow::_loadFiles( QString dir, QString current_file )
{
	g_config.current_dir = dir;
	g_config.last_open_dir = dir;
	QDir myDir( dir );
	QStringList name_filters;
	name_filters << "*.jpg" << "*.png" << "*.bmp"  << "*.tif"  << "*.gif";
	_image_area->setFiles(
		myDir.entryList( name_filters, QDir::Files | QDir::Readable, QDir::Name | QDir::IgnoreCase ),
		current_file );
}
